'use strict';

const types = require('@commitlint/types');
const commitTypes = require('@gitmoji/commit-types');
const gitmojiRegex = require('@gitmoji/gitmoji-regex');
const gitmojis = require('gitmojis');

const gitmojiCodes = gitmojis.gitmojis.map((gitmoji) => gitmoji.code);
const gitmojiUnicode = gitmojis.gitmojis.map((gitmoji) => gitmoji.emoji);
const gitmojiCodeStr = gitmojiRegex.gitmojiCodeRegex.source;
const gitmojiUnicodeStr = gitmojiRegex.gitmojiUnicodeRegex.source;
const emojiStr = gitmojiRegex.emojiRegex.source;
const { Error } = types.RuleConfigSeverity;
const rules = {
  // 使用gitmoji 规则
  "with-gitmoji": [Error, "always"],
  // 使用 Angular 的类型
  "type-enum": [Error, "always", commitTypes],
  // 内容以空行开始
  "body-leading-blank": [Error, "always"],
  // 结尾以空行开始
  "footer-leading-blank": [Error, "always"],
  // 标题最大长度 100 个字符
  "header-max-length": [Error, "always", 100],
  // Scope 永远小写
  "scope-case": [Error, "always", "lower-case"],
  // 不允许标题空着
  "subject-empty": [Error, "never"],
  // 不允许使用句号
  "subject-full-stop": [Error, "never", "."],
  // type 必须小写
  "type-case": [Error, "always", "lower-case"],
  // type 不能为空
  "type-empty": [Error, "never"]
};
const emojiRule = (parsed, _rule, value = "or-emoji") => {
  const { raw } = parsed;
  const gitmojiCodeResult = new RegExp(
    `(${gitmojiRegex.gitmojiCodeRegex.source})\\s.*`,
    "gm"
  ).exec(raw);
  const gitmojiUnicodeResult = new RegExp(
    `(${gitmojiRegex.gitmojiUnicodeRegex.source})\\s.*`,
    "gm"
  ).exec(raw);
  const emojiResult = new RegExp(`(${gitmojiRegex.emojiRegex.source})\\s.*`, "gm").exec(raw);
  let pass;
  let errorMsg = "passed";
  if (gitmojiCodeResult) {
    const emojiCode = gitmojiCodeResult[1];
    pass = gitmojiCodes.includes(emojiCode);
    if (!pass) {
      errorMsg = `${emojiCode} is not in the correct gitmoji list, please check the emoji code on https://gitmoji.dev/.`;
    }
  } else if (gitmojiUnicodeResult) {
    const unicode = gitmojiUnicodeResult[1];
    pass = gitmojiUnicode.includes(unicode);
  } else if (emojiResult) {
    const unicode = emojiResult[1];
    pass = false;
    errorMsg = `${unicode} is not in the correct gitmoji list, please check the emoji code on https://gitmoji.dev/.`;
  } else {
    if (value === "start-emoji") {
      pass = false;
      errorMsg = "Your commit should start with gitmoji code. Please check the emoji code on https://gitmoji.dev/.";
    } else {
      pass = true;
    }
  }
  return [pass, errorMsg];
};
const gitmojiPlugin = {
  rules: {
    "with-gitmoji": emojiRule
  }
};
const parserPreset = {
  parserOpts: {
    headerPattern: new RegExp(
      `^(?:(?:${gitmojiCodeStr}|(?:${gitmojiUnicodeStr})|(?:${emojiStr}))\\s)?(?<type>\\w*)(?:\\((?<scope>.*)\\))?!?:\\s(?<subject>(?:(?!#).)*(?:(?!\\s).))(?:\\s\\(?(?<ticket>#\\d*)\\)?)?$`
    ),
    headerCorrespondence: ["type", "scope", "subject", "ticket"]
  }
  // plugins: {
  //   gitmoji: gitmojiPlugin,
  // },
};
const config = {
  rules,
  parserPreset,
  plugins: [gitmojiPlugin]
};

module.exports = config;
